/// <reference types="node" />
import { EventEmitter } from 'events';
import Command, { GlobalCommand, CommandConfig, HelpCallback, CommandExample } from './Command';
import { OptionConfig } from './Option';
interface ParsedArgv {
    args: ReadonlyArray<string>;
    options: {
        [k: string]: any;
    };
}
interface MriResult extends ParsedArgv {
    rawOptions: {
        [k: string]: any;
    };
}
declare class CAC extends EventEmitter {
    bin: string;
    commands: Command[];
    globalCommand: GlobalCommand;
    matchedCommand: Command;
    /**
     * Raw CLI arguments
     */
    rawArgs: string[];
    /**
     * Parsed CLI arguments
     */
    args: MriResult['args'];
    /**
     * Parsed CLI options, camelCased
     */
    options: MriResult['options'];
    /**
     * Raw CLI options, i.e. not camelcased
     */
    rawOptions: MriResult['rawOptions'];
    constructor();
    /**
     * Add a global usage text.
     *
     * This is not used by sub-commands.
     */
    usage(text: string): this;
    /**
     * Add a sub-command
     */
    command(rawName: string, description: string, config?: CommandConfig): Command;
    /**
     * Add a global CLI option.
     *
     * Which is also applied to sub-commands.
     */
    option(rawName: string, description: string, config?: OptionConfig): this;
    /**
     * Show help message when `-h, --help` flags appear.
     *
     */
    help(callback?: HelpCallback): this;
    /**
     * Show version number when `-v, --version` flags appear.
     *
     */
    version(version: string, customFlags?: string): this;
    /**
     * Add a global example.
     *
     * This example added here will not be used by sub-commands.
     */
    example(example: CommandExample): this;
    /**
     * Output the global help message
     *
     * This will also call `process.exit(0)` to quit the process.
     */
    outputHelp(): void;
    /**
     * Output the version number.
     *
     * This will also call `process.exit(0)` to quit the process.
     */
    outputVersion(): void;
    setParsedInfo({ args, options, rawOptions }: MriResult, matchedCommand?: Command): this;
    /**
     * Parse argv and run command action if found.
     */
    parse(argv?: string[]): ParsedArgv;
    private mri;
    private runCommandAction;
}
export default CAC;
